/*
 * Nutu Bogdan Constantin
 * grupa 336CB
 * Proiect PM 2019
 * Line Follower
 */
#include <avr/io.h>
#include <util/delay.h>
#include "lcd.h"
#include <stdlib.h>
#include <math.h>

//Viteza maxima a celor 2 motoare
#define MAX_SPEED 124

//Valoare folosita ca limita pentru senzorii analogici,
//cu scopul de a determina daca un senzor "vede" alb sau negru
#define BW_LIMIT 300

//Numarul de senzori IR
#define NO_SENSORS 8

#define DELAY_TIME 25

void left_engine_set_speed(int speed)
{
	OCR2A = speed;
}

void right_engine_set_speed(int speed)
{
	OCR0A = speed;
}

void left_engine_init(void)
{
	TCCR2A = 0;
    TCCR2A |= (3 << WGM20) | (1 << COM2A1);
    TCCR2B |= (1 << CS20);

    left_engine_set_speed(MAX_SPEED);
}

void right_engine_init(void)
{
	TCCR0A = 0;
    TCCR0A |= (3 << WGM00) | (1 << COM0A1);
    TCCR0B |= (1 << CS00);

    right_engine_set_speed(MAX_SPEED);
}

void init_engines(void)
{
	left_engine_init();
	right_engine_init();
}

void set_engines_speed(int left_engine_speed , int right_engine_speed)
{
	left_engine_set_speed(left_engine_speed);
	right_engine_set_speed(right_engine_speed);
}

void ADC_init(void)
{
	ADMUX = (1 << REFS0);
	ADCSRA = (1 << ADEN) | (7 << ADPS0);
}

int ADC_get(uint8_t channel)
{
	ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

	ADCSRA |= (1 << ADSC);
	while(ADCSRA & (1 << ADSC));

	return ADC;
}

void init_ports(void)
{
	DDRB |= (1 << PB3);
	PORTB &= ~(1 << PB3);

	DDRD |= (1 << PD7);
	PORTD &= ~(1 << PD7);

	DDRB |= (1 << PB4);
	PORTB |= (1 << PB4);

	DDRB |= (1 << PB5);
	PORTB &= ~(1 << PB5);

	DDRB |= (1 << PB6);
	PORTB &= ~(1 << PB6);

	DDRB |= (1 << PB7);
	PORTB |= (1 << PB7);
}

void run(void) 
{
	int left_black_sensors, right_black_sensors;
	int left_speed, right_speed;
	int i;

	while(1) {
		left_black_sensors = 0;
		right_black_sensors = 0;
		left_speed = MAX_SPEED;
		right_speed = MAX_SPEED;

		for (i = 0; i < NO_SENSORS / 2; i++) {
			if (ADC_get(i) > BW_LIMIT) {
				left_black_sensors += 1;
			}
		}

		for (i = NO_SENSORS / 2; i < NO_SENSORS; i++) {
			if (ADC_get(i) > BW_LIMIT) {
				right_black_sensors += 1;
			}
		}

		if (left_black_sensors > 0 && right_black_sensors > 0) {
			int dif = abs(left_black_sensors - right_black_sensors);

			if (dif == 3 || dif == 4) {
				if (left_black_sensors > right_black_sensors) {
					left_speed *= 0;
					right_speed *= 1.1;
				} else {
					left_speed *= 1.1;
					right_speed *= 0;
				}
			} else if (dif == 1 || dif == 2) {
				if (left_black_sensors > right_black_sensors) {
					left_speed *= (10 - dif) / 10;
				} else {
					right_speed *= (10 - dif) / 10;
				}
			} 
		} else if (left_black_sensors > 0 && right_black_sensors == 0) {
			if (left_black_sensors == 1 || left_black_sensors == 2) {
				left_speed *= 0;
			} else if (left_black_sensors == 3) {
				left_speed *= 0.4;
			} else if (left_black_sensors == 4) {
				left_speed *= 0;
				right_speed *= 1.2;
			}
		} else if (right_black_sensors > 0 && left_black_sensors == 0) {
			if (right_black_sensors == 1 || right_black_sensors == 2) {
				right_speed *= 0;
			} else if(right_black_sensors == 3) {
				right_speed *= 0.4;
			} else if (right_black_sensors == 4) {
				left_speed *= 1.2;
				right_speed *= 0;
			}
		}

		set_engines_speed(left_speed, right_speed);

		_delay_ms(DELAY_TIME);
	}
}

int main(void) {

	ADC_init();

	init_ports();

	init_engines();

	_delay_ms(1000);

	run();

	return 0;
}
